package edu.washington.cac.myuw.servlet;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;

import javax.servlet.http.HttpServletResponse;

import org.apache.ecs.Document;
import org.apache.ecs.Element;
import org.apache.ecs.html.A;
import org.apache.ecs.html.BR;
import org.apache.ecs.html.Font;
import org.apache.ecs.html.H3;
import org.apache.ecs.html.H4;
import org.apache.ecs.html.P;
import org.apache.ecs.html.PRE;

import edu.washington.cac.myuw.servlet.session.InvalidSession;
import edu.washington.cac.myuw.servlet.test.TestHttpServletResponse;

/** 
  static routines to print out error message pages 
  @author Fang Lin, Greg Barnes
  @version 3.2
 */
public class MessageMaker 
{
  /** Name of the application */
  private String appName;

  /** URL of the 'welcome' page */
  private String welcomeUrl;

  /** URL of the 'main' page for the application */
  private String defaultUrl;

  /** An email address to contact if there are problems with the application */
  private String contactEmail;

  /** A URL explaining how to configure a browser to accept cookies */
  private String cookiesHelpUrl;

  public MessageMaker(String appName, 
                      String welcomeUrl, 
                      String defaultUrl, 
                      String contactEmail,
                      String cookiesHelpUrl) 
  {
    this.appName = appName;
    this.welcomeUrl = welcomeUrl;
    this.defaultUrl = defaultUrl;
    this.contactEmail = contactEmail;
    this.cookiesHelpUrl = cookiesHelpUrl;
  }

  /* print a message page */
  private void messagePage(HttpServletResponse res, 
                                      String title, 
                                      String body) 
      throws IOException
  {
    if (res == null) {
      throw new IllegalArgumentException(
          "MessageMaker::messagePage: null parameter.");
    }

    res.setContentType("text/html");
    res.setHeader("Expires","Thu, 01-Jan-1970 00:00:01 GMT");
    res.setHeader("Cache-Control", "no-cache, must-revalidate");
    res.setHeader("Pragma", "no-cache");
    new Document().appendTitle(title).
                   appendBody(body).
                   output(res.getOutputStream());
    res.getOutputStream().close();
  }

  private static final String EXIT_TITLE = "Please Exit!";
  private static final String EXIT_BODY1 = "If you want to enter ";
  private static final String EXIT_BODY2 = 
    ", please exit from this browser and start a new one.";

  public void exitMessage(HttpServletResponse res) throws IOException 
  {
    messagePage(res, EXIT_TITLE, 
        new P().addElement(EXIT_BODY1 + this.appName + EXIT_BODY2).toString());
  }

  private static final String SESSION_TITLE = "Invalid Session Error";
  private static final String SESSION_BODY1 = "Your ";
  private static final String SESSION_BODY2 = " session expired.";
  private static final String SESSION_BODY3 = 
    "If you have logged out, please exit your browser now." + 
    new BR() + new BR() + 
    "If you did not log out, you may ";

  private static final String SESSION_ACTION1 = "go to the ";
  private static final String SESSION_ACTION2 = "Welcome page.";
  private static final String SESSION_BODY4 = 
      "If you continue to see this page, please ";
  private static final String SESSION_BODY5 = 
      "configure your browser to accept cookies from ";
   
  /**
    get an explanation of the action a user can perform if they were 
    thrown out due to an invalid session.  By default, they can
    go back to the welcome page, but this can be overridden.

    @returns an HTML String that can be displayed saying what the
      user can do next
   */
  protected String sessionAction() 
  {
    return SESSION_ACTION1 + new A(this.welcomeUrl, SESSION_ACTION2);
  }
    
  /**
    Get the body of the page a user sees when they have an invalid session
    @return the body of the page a user sees when they have an invalid session
   */
  private String sessionBody() 
  {
    return new H4(SESSION_BODY1 + this.appName + SESSION_BODY2) + "\n" +
        new P().addElement(SESSION_BODY3 + sessionAction() + "\n" +
                           SESSION_BODY4 + 
                           new A(this.cookiesHelpUrl, 
                                 SESSION_BODY5 + this.appName));
  }

  public void sessionErrorMessage(String mes, HttpServletResponse res) 
      throws IOException 
  {
    if (mes != null) {
      messagePage(res, SESSION_TITLE, new H3(mes) + sessionBody());
    } else {
      messagePage(res, SESSION_TITLE, sessionBody().toString());
    }
  }

  public void sessionErrorMessage(HttpServletResponse res) 
      throws IOException 
  {
    sessionErrorMessage(null, res);
  }

  private static final String WARNING_TITLE = "Warning";

  public void warningMessage(HttpServletResponse res, String msg)
      throws IOException 
  {
    messagePage(res, WARNING_TITLE, msg);
  }

  private static final String GENERIC_TITLE = "Error";
  private static final String GENERIC_BODY_PRE1 = 
    " has a problem right now.";
  private static final String GENERIC_BODY_PRE2 = "Please contact ";
  private static final String GENERIC_BODY_PRE3 = 
    " and enclose the contents of this page.  Thanks." + new BR();

  public void genericErrorMessage(HttpServletResponse res, Throwable e)
      throws IOException 
  {
    /** Not to print stack trace to the browser for security reason 
    StringWriter sw = new StringWriter();
    e.printStackTrace(new PrintWriter(sw));
    */
    messagePage(res, GENERIC_TITLE, 
        new H3(this.appName + GENERIC_BODY_PRE1) + "\n" + 
        new P().addElement(GENERIC_BODY_PRE2 + this.contactEmail + 
                           GENERIC_BODY_PRE3) 
        + "\n" + e );
  }

  private static final String DB_TITLE = " Database Error";
  private static final String DB_BODY1 = 
      "There is currently a problem with the ";
  private static final String DB_BODY2 = 
      " database, please try again.";
  private static final String DB_BODY3 = 
      "If you continue to receive this message, please contact ";
  private static final String DB_BODY4 = ".  Thanks.";

  /**
    Emit a page indicating there was a database error
    @param res The HTTP response object
    @exception IOException If we have trouble writing the page
   */
  public void dbErrorMessage(HttpServletResponse res) throws IOException 
  {
    messagePage(res, this.appName + DB_TITLE, 
                new H3(DB_BODY1 + this.appName + DB_BODY2) + "" +
                new P().addElement(DB_BODY3 + this.contactEmail + DB_BODY4));
  }

  private static final String PARAM_TITLE = "FORM Input Error";
  private static final Element PARAM_BODY1 = new H3("Some fatal errors have been found in your FORM inputs from the previous page.");
  private static final String PARAM_BODY2 = "Please email ";
  private static final String PARAM_BODY3 = " and report this error, including a description of what you inputted.  Thanks.  You may go back to ";
  private static final String PARAM_BODY4 = "the main page.";

  public void parameterExceptionMessage(HttpServletResponse res)
      throws IOException 
  {
    messagePage(res, PARAM_TITLE, PARAM_BODY1 + "" + 
        new P().addElement(PARAM_BODY2 + this.contactEmail + PARAM_BODY3 + 
                           new A(this.defaultUrl, PARAM_BODY4)));
  }

  private static final String LOGIN_BODY1 = "Logging you in to ";
  private static final String LOGIN_BODY2 = "To PROTECT YOUR PRIVACY, LOGOUT and EXIT your Web browser when you finish using ";
  private static final String LOGIN_BODY3 = 
      "If your browser is stuck, please click to enter ";

  public void loginMessage(HttpServletResponse res, 
                           String url, 
                           String linktitle)
      throws IOException
  {
    messagePage(res, "", new P().addElement(LOGIN_BODY1 + this.appName).
                addElement(new BR()).
                addElement(new Font().setColor("#FF0000").
                           addElement(LOGIN_BODY2 + this.appName + ".")).
                addElement(new BR()).
                addElement(LOGIN_BODY3).
                addElement(new A (url, linktitle)).
                toString());
  }

  public void loginMessage(HttpServletResponse res, String url)
      throws IOException
  {
    loginMessage(res, url, this.appName);
  }


  private static final String STARS = "\n*************************";

  public static void main(String[] args) 
    throws IOException
  {
    TestHttpServletResponse thsr = new TestHttpServletResponse(System.err);
    MessageMaker mm = new MessageMaker("TEST APP", "WELCOMEURL", 
        "DEFAULTURL", "TESTEMAIL", "COOKIESURL");
    mm.loginMessage(thsr, "DEFAULTURL", "THE NEXT PAGE");
    System.err.println(STARS);
    mm.parameterExceptionMessage(thsr);
    System.err.println(STARS);
    mm.dbErrorMessage(thsr);
    System.err.println(STARS);
    mm.genericErrorMessage(thsr, new InvalidSession("whee"));
    System.err.println(STARS);
    mm.sessionErrorMessage("Auggh", thsr);
    System.err.println(STARS);
    mm.sessionErrorMessage(thsr);
    System.err.println(STARS);
    mm.exitMessage(thsr);
    System.err.println(STARS);
  }
}
