package edu.washington.cac.myuw.servlet;

import java.io.IOException;
import java.io.PrintWriter;
import java.io.StringWriter;
import java.sql.SQLException;

import javax.servlet.http.HttpServletResponse;

import edu.washington.cac.myuw.servlet.session.InvalidSession;

/** useful routines for a servlet 

  @author Greg Barnes
  @version 3.2
*/
public class ServletSupport 
{
  /** Object that can generate a log file */
  private LogFactory logFactory;

  /** The approximate time when the servlet began running */
  private long startTime;

  /** Should we print timing information to the log? */
  private boolean timed;     

  /** Text to print with the timing information */
  private String timedFlags;  

  /** Printable id of the servlet */
  private String id = "";

  /** A log file for the servlet */
  private Log log = null;

  /** Object that can create output web pages */
  protected MessageMaker mm;

  /** Constructor
    @param id Printable id of the Servlet
    @param mm Object that can create output web pages
    @param logFactory Object that can generate a log file
   */
  public ServletSupport(String id, MessageMaker mm, LogFactory logFactory) 
  {
    this(id, mm, logFactory, false);
  }

  /** Constructor
    @param id Printable id of the Servlet
    @param mm Object that can create output web pages
    @param logFactory Object that can generate a log file
    @param timed Should we print timing information to the log?
   */
  public ServletSupport(String id, 
                        MessageMaker mm, 
                        LogFactory logFactory, 
                        boolean timed)
  {
    this(id, mm, logFactory, timed, null); 
  }

  /** Constructor
    @param id Printable id of the Servlet
    @param mm Object that can create output web pages
    @param logFactory Object that can generate a log file
    @param timed Should we print timing information to the log?
    @param timedFlags Text to print with the timing information.  If null,
      nothing is printed
   */
  public ServletSupport(String id, 
                        MessageMaker mm, 
                        LogFactory logFactory,
                        boolean timed, 
                        String timedFlags) 
  {
    this.id = id;
    this.mm = mm;
    this.logFactory = logFactory;
    this.timed = timed;
    this.timedFlags = timedFlags;

    if (timed) {
      startTime = System.currentTimeMillis();
    }
  }

  /** 
    Set the printable id of the servlet
    @param id The new id
   */
  public void setId(String id) 
  {
    this.id = id;
  }

  /** 
    set the log file 
    @param log The Log
   */
  public void setLog(Log log) 
  {
    this.log = log;
    this.log.println(id + " starting...");
  }

  /** 
    Get the log file
    @return the log file
   */
  public Log getLog() 
  {
    return this.log;
  }

  /** 
    Perform necessary tasks when the servlet is done, such a printing
    timing information to the log, if appropriate.
    This method should be called by all servlets that end without an error.
   */
  public void done() 
  {
    if (this.log == null) {
      this.log = this.logFactory.create();
    }

    if (timed) {
      long totalTime = System.currentTimeMillis() - startTime;
      this.log.println(id + " ending. Time= " +
                      (timedFlags == null ? "" : "(" + timedFlags + ") ") +
                      totalTime + " milliseconds.");
    } else {
      this.log.println(id + " ending");
    }
  }

  /* common exceptions that occur when someone clicks before the page is
     completely loaded.  Not really errors */
  private static final String WRITE_ALL_BYTES = 
      "Error could not write all bytes";
  private static final String BROKEN_PIPE = "Broken pipe";
  private static final String NO_PROCESS = 
      "There is no process to read data written to a pipe";

  /* exception that occurs when someone logs out before the page is 
     finished loading */
  private static final String INVALIDATED_SESSION1 = "session is invalid";
  private static final String INVALIDATED_SESSION2 = "has been invalidated.";

  public static boolean isInvalidatedSessionException(Throwable e) 
  {
    return e instanceof IllegalStateException && 
           (e.getMessage().endsWith(INVALIDATED_SESSION1) ||
            e.getMessage().endsWith(INVALIDATED_SESSION2) );
  }

  private static boolean isCommonError(Throwable e) 
  {
    String m = e.getMessage();

    /*
      UnsupportedEncoding implies a bad browser setting.
      Other 3 indicate the client browser is gone. 
     */
    return e instanceof IOException && 
           (e instanceof java.io.UnsupportedEncodingException ||
            (m != null && (m.startsWith(WRITE_ALL_BYTES) || 
             m.startsWith(BROKEN_PIPE) || m.startsWith(NO_PROCESS))));
  }

  protected boolean shouldPrintStackTraceToLog(Throwable e) 
  {
    return !isInvalidatedSessionException(e) && !isCommonError(e);
  }

  /** 
    Print an error to the log
    @param e The error
   */
  protected void printToLog(Throwable e) 
  {
    if (log == null) {
      log = logFactory.create();
    }

    if (!(e instanceof InvalidSession)) {
      if (shouldPrintStackTraceToLog(e)) {
        StringWriter sw = new StringWriter();
        e.printStackTrace(new PrintWriter(sw));
        log.println(id + ": " + sw);
      } else {
        log.println(id + " raised " + e);
      }
    }
  }

  protected boolean shouldPrintToBrowser(Throwable e) 
  {
    return !isInvalidatedSessionException(e) && !isCommonError(e);
  }

  protected void printToBrowser(Throwable e, HttpServletResponse resp) 
      throws IOException
  {
    if (e instanceof InvalidSession) {
      mm.sessionErrorMessage(e.getMessage(), resp);
    } else if (e instanceof SQLException) {
      mm.dbErrorMessage(resp);
    } else if (shouldPrintToBrowser(e)) {
      mm.genericErrorMessage(resp, e);
    }
  }

  /** 
    Handle any exceptions thrown by the servlet.
    This method should be called by all servlets that end with an error.

    @param e The error thrown
    @param resp Http response object
    @throws IOException If there's a problem writing a web page or
       an error message to the log
   */
  public void handleException(Throwable e, HttpServletResponse resp) 
      throws IOException 
  {
    printToLog(e);
    printToBrowser(e, resp);
  }
}
