// java

package uwcal.out.forms;

import java.sql.Time;
import java.util.Calendar;
import java.util.Date;
import java.util.Iterator;

import org.apache.ecs.Element;
import org.apache.ecs.ElementContainer;
import org.apache.ecs.StringElement;
import org.apache.ecs.html.BR;
import org.apache.ecs.html.Input;
import org.apache.ecs.html.Option;
import org.apache.ecs.html.Select;
import org.apache.ecs.html.Span;

import edu.washington.cac.calendar.CalParameter;
import edu.washington.cac.calendar.MyCalendar;
import edu.washington.cac.calendar.data.Event;
import edu.washington.cac.calendar.data.Location;
import edu.washington.cac.calendar.data.Locations;
import edu.washington.cac.calendar.data.User;
import edu.washington.cac.calendar.db.DBResources;
import edu.washington.cac.myuw.util.Coder;

import uwcal.out.DisplayText;
import uwcal.out.Style;
import uwcal.out.Page;
import uwcal.out.Util;

/**
  A form dealing with an event
  @author Greg Barnes
  @version 1.0
 */
public abstract class EventForm extends CalendarForm 
{
  Event e;
  Locations ls;
  
  CommonFormElements cfe;

  /** Create a new instance
    @param page Page this form is a part of
    @param e The event. if it is null, adding new event;
      otherwise, editing old event.
    @param ls The event locations.
  */
  EventForm(Page page, Event e, Locations ls)
  {
    super(page);
    this.e = e;
    this.ls = ls;
    cfe = new CommonFormElements(e);
  }

  protected abstract Element dateTimeHeader();
  protected abstract Element dateTimeDivider();
  protected abstract Element startDateTime();
  protected abstract Element endDateTime();
  protected abstract Element location();

  private static final String[] AMPM_OPTION_STRINGS = {

    FormsDisplayText.AM, FormsDisplayText.PM 
  };

  private static int ampmSelected(Time t) 
  {
    if (t == null || new MyCalendar(t).get(Calendar.AM_PM) == Calendar.PM) {
      return 1;   //"PM"
    } else {
      return 0;   //"AM"
    }
  }

  private static Option[] ampmOptions(Time initialValue) 
  {
    Option[] o = new Option().addElement(AMPM_OPTION_STRINGS);
    o[ampmSelected(initialValue)].setSelected(true);
    return o;
  }

  private static final String[] HOUR_OPTION_STRINGS = { 
      DisplayText.NO_VALUE, 
      "1", "2", "3", "4", "5", "6", "7", "8", "9", "10", "11", "12" 
  };

  private static int hourSelected(Time t) 
  {
    if (t == null) { 
      return 0;   // NO_VALUE
    } else if (new MyCalendar(t).get(Calendar.HOUR) == 0) {
      return 12;
    } else {
      return new MyCalendar(t).get(Calendar.HOUR);
    }
  }

  private static Option[] hourOptions(Time initialValue) 
  {
    Option[] o = new Option().addElement(HOUR_OPTION_STRINGS);
    o[hourSelected(initialValue)].setSelected(true);
    return o;
  }

  private static final String[] MINUTE_OPTION_STRINGS = { 
      "00", "05", "10", "15", "20", "25", "30", "35", "40", "45", "50", "55" 
  };

  private static int minuteSelected(Time t) 
  {
    if (t == null) { 
      return 0;   // NO_VALUE
    } else {
      return new MyCalendar(t).get(Calendar.MINUTE) / 5;
    }
  }

  private static Option[] minuteOptions(Time initialValue) 
  {
    Option[] o = new Option().addElement(MINUTE_OPTION_STRINGS);
    o[minuteSelected(initialValue)].setSelected(true);
    return o;
  }

  private static Element timeSelects(String hourName,
                                     String minuteName, 
                                     String ampmName,
                                     Time initialValue)
  {
    return new ElementContainer().
        addElement(new Select(hourName, hourOptions(initialValue))).
        addElement(":").
        addElement(new Select(minuteName, minuteOptions(initialValue))).
        addElement("\n").
        addElement(new Select(ampmName, ampmOptions(initialValue))).
        addElement("\n");
  }

  private static Element hourInput(String hourName, 
                                   String minuteName, 
                                   String ampmName,
                                   Time initialValue)
  {
    return new ElementContainer().
        addElement(Util.spanItem(FormsDisplayText.AT + " ", Style.STD_TEXT)).
        addElement(timeSelects(hourName, minuteName, ampmName, initialValue));
  }
  
  protected Element startHourInput(Time selectedTime) 
  {
    return hourInput(CalParameter.START_HOUR, CalParameter.START_MINUTE, 
                     CalParameter.START_AMPM, selectedTime);
  }

  protected Element endHourInput(Time selectedTime) 
  {
    return hourInput(CalParameter.END_HOUR, CalParameter.END_MINUTE, 
                     CalParameter.END_AMPM, selectedTime);
  }

  protected Option[] locationOptions(int locationid) 
  {
    Option[] os = new Option[ls.size()];
    int i;
    Iterator locations = ls.sortedElements();

    for (i = 0; i < os.length && locations.hasNext(); i++) {
      Location l = (Location) locations.next();

      if (l.getCreator().equals(this.page.getUser()) || 
          l.getCreator().equals(DBResources.specialCreator())) 
      {
        os[i] = new Option(l.getId() + "");
        os[i].addElement(Coder.HTMLencode(l.toString()));
        os[i].setSelected(l.getId() == locationid);
      }
    }

    return os;
  }

  protected Element locationTextbox() 
  {
    return new Input(Input.TEXT, CalParameter.ADDRESS, "").setSize(25);
  }

  protected Element locationInput(int locationid) 
  {
    return new ElementContainer().
        addElement(Util.spanItem(FormsDisplayText.USE_OLD_LOCATION,
                                 Style.STD_TEXT)).
        addElement(new Select(CalParameter.LOCATION_ID, 
                              locationOptions(locationid))).
        addElement("\n").addElement(new BR()).
        addElement(new Span().
            addElement(Util.spanItem(FormsDisplayText.OR, Style.BOLD)).
            addElement(" " + FormsDisplayText.ADD_NEW_LOCATION). 
            setClass(Style.STD_TEXT)).
        addElement("\n").
        addElement(locationTextbox()).
        addElement("\n");
  }

  protected Element dateValue(MyCalendar mc) 
  {
    return new StringElement(mc.dateString());
  }

  protected Element startDateInput(Date selectedDate) 
  {
    return new DateInputs(getMc(), CalParameter.START_MONTH, 
                          CalParameter.START_DAY, CalParameter.START_YEAR, 
                          selectedDate, new NoSameWidget()).getInputs();
  }

  protected Element endDateInput(Date endDate, Date startDate) 
  {
    return new DateInputs(getMc(), CalParameter.END_MONTH, 
                          CalParameter.END_DAY, 
                          CalParameter.END_YEAR, endDate, 
                          new EndSameWidget(startDate, endDate)).getInputs();
  }

  protected Element dateTimeElement() 
  {
    return new ElementContainer().addElement(dateTimeHeader()).
        addElement(startDateTime()).addElement(dateTimeDivider()).
        addElement(endDateTime());
  }

  protected Element[] inputElements() 
  {
    Element[] e = { cfe.shortdesc(), cfe.link(FormsDisplayText.EVENT_LINK),
                    location(), dateTimeElement()};
    return e;
  }

  protected String submitButtonText() 
  {
    return FormsDisplayText.ADD_EVENT;
  }
}
